# -*- coding: utf-8 -*-

from django.db.models import Q

from kinder.core.dict.models import DouType
from kinder.core.emie_unit.models import UnitDocFiles
from kinder.core.group.models import Group
from kinder.core.group.models import GroupStatusEnum
from kinder.core.group.models import Pupil
from kinder.core.unit.models import FilialData
from kinder.core.unit.models import StructureTypes
from kinder.core.unit.models import UnitStatus

from .utils import UnitHelper


ORGANIZATION_TYPES = {
    DouType.MUNICIPAL: 1,
    DouType.GOVERNMENT: 2,
    DouType.NOT_GOVERNMENT_WITH_LICENSE: 3,
    DouType.NOT_GOVERNMENT_WITHOUT_LICENSE: 4,
    DouType.IP_WITH_LICENSE: 5,
    DouType.IP_WITHOUT_LICENSE: 6,
    DouType.DEPARTMENTAL: 7
}


def get_dou_type(dou):

    dou_type = dou.dou_type
    if dou_type is None or dou_type.code == DouType.OTHER:
        code = DouType.GOVERNMENT
    else:
        code = dou_type.code
    return ORGANIZATION_TYPES[code]


WORK = 1
RECONSTRUCTION = 3
IN_CAPITAL_REPAIR = 2
SUSPENDED = 4
NO_CONTINGENT = 5
PENDING_OPEN = 6

STATUSES = {
    UnitStatus.RECONSTRUCTION: RECONSTRUCTION,
    UnitStatus.IN_CAPITAL_REPAIR: IN_CAPITAL_REPAIR,
    UnitStatus.SUSPENDED: SUSPENDED,
    UnitStatus.PENDING_OPEN: PENDING_OPEN
}

# статусы при которых в случае отсутствия детей в ДОУ
# устанавливаются нулевые свободные места
NOT_CALCULATED_STATUSES = (
    RECONSTRUCTION, IN_CAPITAL_REPAIR, SUSPENDED, PENDING_OPEN
)


def _is_working_dou(dou):
    u"""Проверяет является ли функционирующим ДОУ.

    Функционирующим считается ДОУ у которого (либо у филиалов которого)
    есть дети в плановых / фактических группах.
    """
    result = False

    filials = FilialData.objects.filter(
        head=dou
    ).filter(
        UnitHelper.ALLOWED_FILIAL_FILTER
    ).values_list("filial_id", flat=True).iterator()

    dou_groups = Group.objects.filter(
        status=GroupStatusEnum.FACT
    ).filter(
        Q(unit=dou) |
        Q(unit__in=filials)
    )

    for group in dou_groups:
        children_in_fact = Pupil.objects.filter(
            grup=group, temporary_deduct=False,
            grup__status=GroupStatusEnum.FACT
        )

        children_in_plan = Pupil.objects.filter(
            Q(grup__room_id__isnull=False, grup__room_id=group.room_id)
        ).filter(
            temporary_deduct=False, grup__status=GroupStatusEnum.PLAN
        ).exclude(
            children_id__in=Pupil.objects.filter(
                grup__unit=group.unit, temporary_deduct=False,
                grup__status=GroupStatusEnum.FACT
            ).values_list("children_id", flat=True)
        )

        if children_in_fact.exists() or children_in_plan.exists():
            result = True
            break

    return result


def get_dou_status(dou):
    if _is_working_dou(dou):
        status = WORK
    elif dou.status == UnitStatus.RECONSTRUCTION:
        status = STATUSES[UnitStatus.RECONSTRUCTION]
    elif dou.status == UnitStatus.IN_CAPITAL_REPAIR:
        status = STATUSES[UnitStatus.IN_CAPITAL_REPAIR]
    elif dou.status == UnitStatus.SUSPENDED:
        status = STATUSES[UnitStatus.SUSPENDED]
    elif dou.status == UnitStatus.PENDING_OPEN:
        status = STATUSES[UnitStatus.PENDING_OPEN]
    else:
        status = NO_CONTINGENT

    return status


def get_dou_structure_type(dou):

    conformity = {
        StructureTypes.KINDER: 1,
        StructureTypes.SCHOOL: 2,
        StructureTypes.PRESOTHER: 3,
        StructureTypes.UNIVERSITY: 4,
    }

    if dou.structure_type == StructureTypes.OTHER:
        return 1
    else:
        return conformity[dou.structure_type]


def get_groups_num(dou):

    filials = FilialData.objects.filter(
        head=dou
    ).filter(
        UnitHelper.ALLOWED_FILIAL_FILTER
    ).values_list("filial_id", flat=True).iterator()

    dou_groups = Group.extended_objects.with_count_norm().select_related(
        'type', 'health_need'
    ).filter(
        status=GroupStatusEnum.FACT
    ).filter(
        Q(unit=dou) | Q(unit__in=filials)
    )

    return dou_groups.count()


def has_fact_groups(dou):
    u"""Проверяет есть ли в ДОУ фактические группы"""

    return Group.objects.filter(
        unit=dou,
        status=GroupStatusEnum.FACT
    ).exists()


def get_dou_filials(dou):
    u"""Возвращает филиалы ДОУ"""

    return FilialData.objects.filter(
        head=dou
    ).filter(UnitHelper.ALLOWED_FILIAL_FILTER)


def get_filial_num(dou):
    u"""Количество подтвержденных филиалов."""

    filials = get_dou_filials(dou).select_related('filial')

    filials_count = 0
    for filial_data in filials:
        if has_fact_groups(filial_data.filial):
            filials_count += 1

    return filials_count


def get_unit_fias_house_guid(dou):
    u"""Возвращаем фиас код адреса организации."""

    if dou.address_house_guid:
        return dou.address_house_guid
    elif dou.address_street:
        return dou.address_street
    elif dou.address_place:
        return dou.address_place
    else:
        return "00000000-0000-0000-0000-000000000000"


def get_additional_info_by_dou(dou, model=UnitDocFiles):
    u"""Дополнительная информация по организации."""

    filials = UnitHelper.get_filial(dou)
    extensions = model.objects.filter(
        Q(unit=dou) | Q(unit__in=filials)
    ).values_list('name', flat=True)

    if not extensions.exists():
        return u'нет'
    else:
        filled_extensions = [
            extension for extension in extensions
            if extension
        ]
        info = ','.join(filled_extensions)
        if info:
            return info
        else:
            return u'нет'


def get_filial_info(dou):

    return 1 if UnitHelper.get_filial(dou).exists() else 0
